PROGRAM_NAME='nx_ip_client_application'
(***********************************************************)
(*  FILE CREATED ON: 05/11/2004  AT: 11:54:59              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 11/11/2004  AT: 10:03:03        *)
(***********************************************************)
(*{{PS_SOURCE_INFO(PROGRAM STATS)                          *)
(*  ORPHAN_FILE_PLATFORM: 1                                *)
(*!!FILE REVISION:                                         *)
(*  REVISION DATE: 08/13/2002                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*                                                         *)
(***********************************************************)
(*}}PS_SOURCE_INFO                                         *)

(***********************************************************)
(* IMPORTANT !                                             *)
(*                                                         *) 
(*  The following code blocks are provided as a guide to   *)
(*  programming the device(s). This is a                   *)
(*  sample only, and will most likely require modification *)
(*  to be integrated into a master program.                *)
(*                                                         *)
(*  Device-specific protocols should be obtained directly  *)
(*  from the equipment manufacturer to ensure compliance   *)
(*  with the most current versions. Within the limits      *)
(*  imposed by any non-disclosure agreements, AMX will     *)
(*  provide protocols upon request.                        *)
(*                                                         *)
(*  If further programming assistance is required, please  *)
(*  contact your AMX customer support team.                *)
(***********************************************************)
DEFINE_DEVICE
dvIP_CLIENT    =    0:5:0  // IP Communication Device
dvPANEL        =  128:1:0  // TOuchpanel


(***********************************************************)
DEFINE_VARIABLE
CHAR sIP_CLIENT_BUFFER[20000] // for receiving data sent from a server
INTEGER CONNECTED           // set if IP connection was established


(***********************************************************)
// DEFINE_FUNCTION GET_IP_ERROR
// PARAMETER:    
// lERR - ERROR NUMBER
// RETURNS: 
// ERROR STRING
DEFINE_FUNCTION CHAR[100] GET_IP_ERROR (LONG lERR)
{
  SWITCH (lERR)
  {
    CASE 0:
      RETURN "";
    CASE 2:
      RETURN "'IP ERROR (',ITOA(lERR),'): General Failure (IP_CLIENT_OPEN/IP_SERVER_OPEN)'";
    CASE 4:
      RETURN "'IP ERROR (',ITOA(lERR),'): unknown host (IP_CLIENT_OPEN)'";
    CASE 6:
      RETURN "'IP ERROR (',ITOA(lERR),'): connection refused (IP_CLIENT_OPEN)'";
    CASE 7:
      RETURN "'IP ERROR (',ITOA(lERR),'): connection timed out (IP_CLIENT_OPEN)'";
    CASE 8:
      RETURN "'IP ERROR (',ITOA(lERR),'): unknown connection error (IP_CLIENT_OPEN)'";
    CASE 9: // not used
    {
      //RETURN "'IP ERROR (',ITOA(lERR),'): Already closed (IP_CLIENT_CLOSE/IP_SERVER_CLOSE)'";  
    }
    CASE 10:
      RETURN "'IP ERROR (',ITOA(lERR),'): Binding error (IP_SERVER_OPEN)'";
    CASE 11:
      RETURN "'IP ERROR (',ITOA(lERR),'): Listening error (IP_SERVER_OPEN)'";  
    CASE 14: // not used
    {
      //RETURN "'IP ERROR (',ITOA(lERR),'): local port already used (IP_CLIENT_OPEN/IP_SERVER_OPEN)'";
    }
    CASE 15:
      RETURN "'IP ERROR (',ITOA(lERR),'): UDP socket already listening (IP_SERVER_OPEN)'";
    CASE 16:
      RETURN "'IP ERROR (',ITOA(lERR),'): too many open sockets (IP_CLIENT_OPEN/IP_SERVER_OPEN)'";
    CASE 17:
      RETURN "'IP ERROR (',ITOA(lERR),'): Local port not open'";
    DEFAULT:
      RETURN "'IP ERROR (',ITOA(lERR),'): Unknown'";
  }
}

// DEFINE_FUNCTION DEV_TO_STRING
// PARAMETER: 
// dvDEV - DEVICE USED
// RETURNS:
// dvDEV AS A STRING
DEFINE_FUNCTION CHAR[17] DEV_TO_STRING (DEV dvDEV)
{
  RETURN "FORMAT('%d',dvDEV.NUMBER),':',FORMAT('%d',dvDEV.PORT),':',FORMAT('%d',dvDEV.SYSTEM)"
}



(***********************************************************)
DEFINE_START
CREATE_BUFFER dvIP_CLIENT,sIP_CLIENT_BUFFER

(***********************************************************)
DEFINE_EVENT
DATA_EVENT[dvIP_CLIENT]
{
  ONLINE: // IP Device comes ONLINE, if connection was established on IP level
  {
    CONNECTED = 1
  }
  OFFLINE: // IP Device goes OFFLINE, if connection was closed on IP level 
  {
    CONNECTED = 0
  }
  STRING:
  {
    // operate received data like data of a serial device
  }
  ONERROR: // if an error message comes from IP Connection
  {
    IF(DATA.Number <> 0) // DATA.Number=0 -> no error
    {
      SEND_STRING 0,"'Device ',DEV_TO_STRING(DATA.Device),':',GET_IP_ERROR(DATA.Number),13,10"
    }

  }
}


BUTTON_EVENT[dvPANEL,1] // Go Online
{
  PUSH:
  { // IP_CLIENT_OPEN(<unique ID>,<URL>,<IP port>,<Protocol>)
    // Protocol: 1=TCP, 2=UDP, 3=Multicast
    IP_CLIENT_OPEN(dvIP_CLIENT.Port,'192.168.0.1',23,1)
  }
}

BUTTON_EVENT[dvPANEL,2] // Go Offline
{
  PUSH:
  {
    IP_CLIENT_CLOSE(dvIP_CLIENT.Port)
  }
}


BUTTON_EVENT[dvPANEL,3] // send a string, depending on the protocol language of the server
{
  PUSH:
  {
    IF(CONNECTED)
    {
      SEND_STRING dvIP_CLIENT,"'Hello World',13,10"
      SEND_STRING dvIP_CLIENT,"$31,$32,$33"
      // this is a string in HTTP language
      SEND_STRING dvIP_CLIENT,"'GET /cgi-bin/dial?rc=@&A=D1&rd=status http/1.1',13,10,13,10,'Host: 192.168.0.254',13,10,13,10"
    }
  }
}

(***********************************************************)
DEFINE_PROGRAM