PROGRAM_NAME='nx_sony-rz30p_basics'
(***********************************************************)
(*  FILE CREATED ON: 11/19/2004  AT: 13:36:13              *)
(***********************************************************)
(*{{PS_SOURCE_INFO(PROGRAM STATS)                          *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 11/22/2004  AT: 11:16:24        *)
(***********************************************************)
(*  ORPHAN_FILE_PLATFORM: 1                                *)
(*!!FILE REVISION:                                         *)
(*  REVISION DATE: 08/13/2002                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*                                                         *)
(***********************************************************)
(*}}PS_SOURCE_INFO                                         *)

(***********************************************************)
(* IMPORTANT !                                             *)
(*                                                         *) 
(*  The following code blocks are provided as a guide to   *)
(*  programming the device(s). This is a                   *)
(*  sample only, and will most likely require modification *)
(*  to be integrated into a master program.                *)
(*                                                         *)
(*  Device-specific protocols should be obtained directly  *)
(*  from the equipment manufacturer to ensure compliance   *)
(*  with the most current versions. Within the limits      *)
(*  imposed by any non-disclosure agreements, AMX will     *)
(*  provide protocols upon request.                        *)
(*                                                         *)
(*  If further programming assistance is required, please  *)
(*  contact your AMX customer support team.                *)
(***********************************************************)

DEFINE_DEVICE
dvDEBUG 		= 0:1:0
dvSONYCAM 	= 0:5:0
dvPANEL 		= 128:1:0

//********************************
DEFINE_VARIABLE
CHAR sCAM_IP_ADDRESS[30]	= { '172.16.100.19' } // IP Address/URL of Cam
INTEGER nCAM_IP_PORT 			= 80									// IP Port of Cam
INTEGER nCAM_IP_PROTOCOL	= 1										// IP Protocol of the cam -> TCP

INTEGER nCAM_ONLINE 			= 0										// set if connection is online
CHAR sCAM_BUFFER[20000]		


// { up,dn,lf,rt,stop,ul,ur,dl,dr,homes }
INTEGER nBTN_CAMCONTROL[] = { 21,22,23,24,25,26,27,28,29,30 }


(***********************************************************)
// DEFINE_FUNCTION GET_IP_ERROR
// PARAMETER:    
// lERR - ERROR NUMBER
// RETURNS: 
// ERROR STRING
DEFINE_FUNCTION CHAR[100] GET_IP_ERROR (LONG lERR)
{
  SWITCH (lERR)
  {
    CASE 0:
      RETURN "";
    CASE 2:
      RETURN "'IP ERROR (',ITOA(lERR),'): General Failure (IP_CLIENT_OPEN/IP_SERVER_OPEN)'";
    CASE 4:
      RETURN "'IP ERROR (',ITOA(lERR),'): unknown host (IP_CLIENT_OPEN)'";
    CASE 6:
      RETURN "'IP ERROR (',ITOA(lERR),'): connection refused (IP_CLIENT_OPEN)'";
    CASE 7:
      RETURN "'IP ERROR (',ITOA(lERR),'): connection timed out (IP_CLIENT_OPEN)'";
    CASE 8:
      RETURN "'IP ERROR (',ITOA(lERR),'): unknown connection error (IP_CLIENT_OPEN)'";
    CASE 9: // not used
    {
      //RETURN "'IP ERROR (',ITOA(lERR),'): Already closed (IP_CLIENT_CLOSE/IP_SERVER_CLOSE)'";  
    }
    CASE 10:
      RETURN "'IP ERROR (',ITOA(lERR),'): Binding error (IP_SERVER_OPEN)'";
    CASE 11:
      RETURN "'IP ERROR (',ITOA(lERR),'): Listening error (IP_SERVER_OPEN)'";  
    CASE 14: // not used
    {
      //RETURN "'IP ERROR (',ITOA(lERR),'): local port already used (IP_CLIENT_OPEN/IP_SERVER_OPEN)'";
    }
    CASE 15:
      RETURN "'IP ERROR (',ITOA(lERR),'): UDP socket already listening (IP_SERVER_OPEN)'";
    CASE 16:
      RETURN "'IP ERROR (',ITOA(lERR),'): too many open sockets (IP_CLIENT_OPEN/IP_SERVER_OPEN)'";
    CASE 17:
      RETURN "'IP ERROR (',ITOA(lERR),'): Local port not open'";
    DEFAULT:
      RETURN "'IP ERROR (',ITOA(lERR),'): Unknown'";
  }
}

// DEFINE_FUNCTION DEV_TO_STRING
// PARAMETER: 
// dvDEV - DEVICE USED
// RETURNS:
// dvDEV AS A STRING
DEFINE_FUNCTION CHAR[17] DEV_TO_STRING (DEV dvDEV)
{
  RETURN "FORMAT('%d',dvDEV.NUMBER),':',FORMAT('%d',dvDEV.PORT),':',FORMAT('%d',dvDEV.SYSTEM)"
}


// This function will create the HTTP GET string, depending on the direction
DEFINE_FUNCTION CHAR[200] SONYCAM_PANTILT_COMMAND(CHAR sIPADR[20],		// IP Address/URL of the Camera
																									INTEGER nDIR,				// Direction to move
																									INTEGER nPSPEED,		// Pan Speed
																									INTEGER nTSPEED)		// Tilt Speed
{
	STACK_VAR CHAR sTEMP[1000], sTEMP2[200]
	STACK_VAR INTEGER nDIR_UNKNOWN
	SWITCH(nDIR) // what direction?
	{	// the VISCA command string is created into sTEMP2 
		CASE 1: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0301FF'" } // Up
		CASE 2: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0302FF'" } // Down
		CASE 3: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0103FF'" } // Left
		CASE 4: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0203FF'" } // Right
		CASE 6: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0101FF'" } // UpLeft
		CASE 7: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0201FF'" } // UpRight
		CASE 8: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0202FF'" } // DownLeft
		CASE 9: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0202FF'" } // DownRight
		CASE 5: 	{ sTEMP2 = "'81010601',FORMAT('%02X',nPSPEED),FORMAT('%02X',nTSPEED),'0303FF'" } // Stop
		CASE 10: 	{ sTEMP2 = "'81010604FF'" } // Home
		DEFAULT:	{ nDIR_UNKNOWN = 1 }
	}
	IF(!nDIR_UNKNOWN) // if direction is ok
	{
		// Now let's put everything together to a HTTP GET string
		sTEMP = "sTEMP,'GET /command/ptzf.cgi?visca=',sTEMP2,' HTTP/1.1',13,10"
		sTEMP = "sTEMP,'Host: ',sIPADR,13,10"
		sTEMP = "sTEMP,'Connection: Keep-Alive',13,10"
		sTEMP = "sTEMP,'Cache-Control: no-cache',13,10"
		sTEMP = "sTEMP,13,10"
		RETURN sTEMP // return the String
	}
}



//********************************
DEFINE_START


DEFINE_EVENT
//********************************

DATA_EVENT[dvSONYCAM]
{
	ONLINE:
	{
		nCAM_ONLINE = 1 // connected to the cam
(* This only was a try...
		SEND_STRING DATA.DEVICE,"'GET /command/ptzf.cgi?visca=81010604FF HTTP/1.1',13,10"
		SEND_STRING DATA.DEVICE,"'Host: ',sCAM_IP_ADDRESS,13,10"
		SEND_STRING DATA.DEVICE,"'Connection: Keep-Alive',13,10"
		SEND_STRING DATA.DEVICE,"'Cache-Control: no-cache',13,10"
		//SEND_STRING DATA.DEVICE,"'Content-Length: 18',13,10"
		SEND_STRING DATA.DEVICE,"13,10"
*)
	}
	OFFLINE:
	{
		nCAM_ONLINE = 0 // connection lost
	}
	STRING:
	{
		// This was sent by the Cam
		SEND_STRING dvDEBUG,"13,10,'----------',13,10"
		SEND_STRING dvDEBUG,"'DATA.TEXT from Cam: ',13,10"
		SEND_STRING dvDEBUG,"DATA.Text"
		SEND_STRING dvDEBUG,"'----------',13,10"
	}
	ONERROR:
	{
		IF(DATA.Number <> 0) // DATA.Number=0 -> no error
    {
      SEND_STRING dvDEBUG,"'Device ',DEV_TO_STRING(DATA.Device),':',GET_IP_ERROR(DATA.Number),13,10"
    }
	}
}


BUTTON_EVENT[dvPANEL,nBTN_CAMCONTROL]
{
	PUSH:
	{
		IF(GET_LAST(nBTN_CAMCONTROL) <> 5) // If not Function STOP
		{
			TO[BUTTON.INPUT]
			IF(nCAM_ONLINE = 0) // if not connected
			{
				// connect to cam
				IP_CLIENT_OPEN(dvSONYCAM.Port,sCAM_IP_ADDRESS,nCAM_IP_PORT,nCAM_IP_PROTOCOL)
			}
			WAIT_UNTIL(nCAM_ONLINE) // wait until the cam is online
			{
				SEND_STRING dvSONYCAM,"SONYCAM_PANTILT_COMMAND(sCAM_IP_ADDRESS,GET_LAST(nBTN_CAMCONTROL),3,3)"
				//*** Debug Output ***
				SEND_STRING dvDEBUG,"'----------',13,10"
				SEND_STRING dvDEBUG,"'To CAM:',13,10"
				SEND_STRING dvDEBUG,"SONYCAM_PANTILT_COMMAND(sCAM_IP_ADDRESS,GET_LAST(nBTN_CAMCONTROL),3,3)"
				SEND_STRING dvDEBUG,"'----------',13,10"
				//********************
			}
		}
	}
	RELEASE:
	{
		IF(nCAM_ONLINE = 0) // if not connected to cam
		{
			// connect
			IP_CLIENT_OPEN(dvSONYCAM.Port,sCAM_IP_ADDRESS,nCAM_IP_PORT,nCAM_IP_PROTOCOL)
		}
		WAIT_UNTIL(nCAM_ONLINE) // wait until connected
		{
			SEND_STRING dvSONYCAM,"SONYCAM_PANTILT_COMMAND(sCAM_IP_ADDRESS,5,3,3)" // Stop
			//*** Debug Output ***
			SEND_STRING dvDEBUG,"'----------',13,10"
			SEND_STRING dvDEBUG,"'To CAM:',13,10"
			SEND_STRING dvDEBUG,"SONYCAM_PANTILT_COMMAND(sCAM_IP_ADDRESS,5,3,3)"
			SEND_STRING dvDEBUG,"'----------',13,10"
			//********************
		}
	}
}


//********************************
DEFINE_PROGRAM
[dvPANEL,1] = (nCAM_ONLINE = 1) // Buttoin Feedback ON if connected to cam









